/*
 * @(#) 1.1 SSOA/ws/code/soa.sca.container.security/src/com/ibm/wsspi/container/security/AccessManager.java, WAS.security.sca, SCA101.SSOA, cf051018.03 7/10/09 17:35:28 [5/10/10 22:21:11]
 *
 * COMPONENT_NAME:  WAS.soa.sca.container.spi
 *
 * Product 5724-J08, 5724-I63, 5724-H88, 5724-H89, 5655-N01, 5722-WE2
 * (C) COPYRIGHT International Business Machines Corp., 2007, 2008
 * All Rights Reserved * Licensed Materials - Property of IBM
 *
 *
 * HISTORY
 * CMVC Ref         Date      Who       Description
 * --------------- 	--------- --------- --------------------------------------------
 * F00CmnContainer  20090626  gregd1    Initial Implementation
 */

package com.ibm.wsspi.container.security;

import java.security.PrivilegedActionException;
import java.security.PrivilegedExceptionAction;
import javax.security.auth.Subject;

/**
 * The AccessManager provides application security role functions.
 *
 * @ibm-spi
 */

public interface AccessManager
{

    /**
     *  The pushApplicationContext method signals a flow of control into the
     *  specified application.  This provides context for security-related processing
     *  on the current thread of execution.  A subsequent call to popApplicationContext
     *  is required when the application returns control.
     *
     *  @param appName - name of the application
     *  @return boolean true if push was completed 
     */
    public boolean pushApplicationContext(String appName);

    /**
     *  The popApplicationContext method signals a return of control from the
     *  specified application.  This provides context for security-related processing
     *  on the current thread of execution.
     *
     *  @param appName - name of the application
     *  @throws IllegalArgumentException if the application name does not match the one
     *  used on the preceding pushApplicationContext call
     *  @throws EmptyStackException if there is no active application context on the thread
     *  @return void  
     */
    public void popApplicationContext(String appName);

    /**
     *  The checkAccess method determines if a given subject has access to any of
     *  a given set of roles within the context of a given application.
     *  
     *  The caller passes an array of role names to be checked (rolesAllowed).
     *  These roles must be a subset of the roles defined in the named application.
     *
     *  The caller also can pass flags indicating to deny or permit access to all
     *  users (isDenyAll, isPermitAll).  If either flag is true, rolesAllowed is
     *  ignored and the flags are used to determine access.  If both flags are
     *  true, then isDenyAll takes precedence.
     *
     *  The resourceName, methodName, and resourceType arguments are used when
     *  auditing is active to audit the check that was performed and its outcome.
     *
     *  @param appName      - name of the application
     *  @param resourceName - name of the target object, for example a bean name
     *  @param methodName   - name of the method invoked on the target object
     *  @param resourceType - identifier that describes the type of resource
     *  @param rolesAllowed - array of role names to be checked for permission
     *  @param isDenyAll    - indicates whether to always deny access
     *  @param isPermitAll  - indicates whether to always permit access
     *  @param subject      - holds the credentials of the principal to be verified
     *  
     *  @throws AccessException Thrown if access is denied
     *  @return void  
     */

    public void checkAccess(
                  String   appName,
                  String   resourceName,
                  String   methodName,
                  String   resourceType,
                  String[] rolesAllowed,
                  boolean  isDenyAll,
                  boolean  isPermitAll,
                  Subject  subject)
           throws AccessException;

    /**
     *  The runAs method performs the given PrivilegedExceptionAction with the
     *  Java thread identity set to the user associated with a given runAs role.
     *  
     *  The caller passes an application name and a role name.  The role name
     *  must be defined as a runAs role in the application.
     *
     *  The resourceName and methodName arguments are used when auditing is
     *  active to audit the use of the runAs role.
     *
     *  @param appName      - name of the application
     *  @param resourceName - name of the target object, for example a bean name
     *  @param methodName   - name of the method invoked on the target object
     *  @param runAsRole    - name of RunAs role
     *  @param action       - the action to perform
     *  
     *  @throws PrivilegedActionException if the action's run method threw a checked exception
     *  @throws DelegationException if there is an error setting the thread identity from the runAs role
     *  @return the value returned by the action's run method 
     */

    public Object runAs(
                  String   appName,
                  String   resourceName,
                  String   methodName,
                  String   runAsRole,
                  PrivilegedExceptionAction action)
           throws PrivilegedActionException, DelegationException;

}